"""
Agnocomplete Widgets
"""
from distutils.version import StrictVersion

from django import get_version

from django.forms import widgets
from django.urls import reverse_lazy
from django.utils.encoding import force_text as text
from django.conf import settings


from . import get_namespace
from .constants import AGNOCOMPLETE_DATA_ATTRIBUTE

__all__ = [
    'AgnocompleteSelect',
    'AgnocompleteTextInput',
    'AgnocompleteMultiSelect'
]


class _AgnocompleteWidgetMixin(object):
    """
    Generic toolset for building Agnocomplete-ready widgets
    """
    def _agnocomplete_build_attrs(self, attrs):
        data_url = reverse_lazy(
            '{}:agnocomplete'.format(get_namespace()),
            args=[self.agnocomplete.slug]
        )
        # Priority to the configurable URL
        data_url = self.agnocomplete.get_url() or data_url
        # Data attribute
        data_attribute = getattr(
            settings, 'AGNOCOMPLETE_DATA_ATTRIBUTE',
            AGNOCOMPLETE_DATA_ATTRIBUTE)
        data_attribute = 'data-{}'.format(data_attribute)
        attrs.update({
            'data-url': data_url,
            'data-query-size': self.agnocomplete.get_query_size(),
            data_attribute: 'on',
        })

        return attrs


if StrictVersion(get_version()) < StrictVersion('1.11'):
    class AgnocompleteWidgetMixin(_AgnocompleteWidgetMixin):
        """
        Generic toolset for building Agnocomplete-ready widgets
        """
        def build_attrs(self, extra_attrs=None, **kwargs):
            attrs = super(AgnocompleteWidgetMixin, self).build_attrs(
                extra_attrs, **kwargs)
            return self._agnocomplete_build_attrs(attrs)

        def render_options(self, *args):
            # Django >= 1.10, only "selected_choices" in the arg list
            if len(args) == 1:
                selected_choices = args[0]
            else:
                # Django < 1.10 - selected_choices is the second arg.
                _, selected_choices = args
            selected_choices = set(text(v) for v in selected_choices)
            selected_choices_tuples = self.agnocomplete.selected(
                selected_choices)
            output = []
            for option_value, option_label in selected_choices_tuples:
                output.append(self.render_option(
                    selected_choices, option_value, option_label))
            return '\n'.join(output)
else:
    class AgnocompleteWidgetMixin(_AgnocompleteWidgetMixin):
        """
        Generic toolset for building Agnocomplete-ready widgets
        """
        def build_attrs(self, base_attrs, extra_attrs=None):
            attrs = super(AgnocompleteWidgetMixin, self).build_attrs(
                base_attrs, extra_attrs)
            return self._agnocomplete_build_attrs(attrs)

        """
        Render only selected options
        """
        def optgroups(self, name, value, attrs=None):
            selected_ids = set(text(v) for v in value)
            selected_choices = self.agnocomplete.selected(selected_ids)
            options = []
            groups = [
                (None, options, 0)  # single unnamed group
            ]

            for option_value, option_label in selected_choices:
                opt = self.create_option(
                    name, option_value, option_label, True, 0,
                    subindex=None, attrs=attrs,
                )
                opt['attrs']['selected'] = True
                options.append(opt)

            return groups


class AgnocompleteSelect(AgnocompleteWidgetMixin, widgets.Select):
    """
    The default Agnocomplete-ready input is a Select box.
    """
    pass


class AgnocompleteTextInput(AgnocompleteWidgetMixin, widgets.TextInput):
    """
    Alternate Agnocomplete-ready widget: TextInput.

    This widget is needed for front librairies which want a text input.
    """
    pass


class AgnocompleteMultiSelect(AgnocompleteWidgetMixin, widgets.SelectMultiple):
    """
    A multi-selection-ready Select widget Mixin
    """
    def __init__(self, create=False, *args, **kwargs):
        super(AgnocompleteMultiSelect, self).__init__(*args, **kwargs)
        self.create = create

    def _agnocomplete_build_attrs(self, attrs):
        attrs = super(AgnocompleteMultiSelect, self). \
            _agnocomplete_build_attrs(attrs)

        if self.create:
            attrs.update({'data-create': 'on'})

        return attrs



"""
Agnocomplete, the Agnostic Autocomplete Django app.

"""
import logging

from django.conf import settings
from django.utils.module_loading import autodiscover_modules

logger = logging.getLogger(__name__)


def get_namespace():
    """
    Return the agnocomplete view namespace.

    Default value is "agnocomplete", but it can be overridden using the
    ``AGNOCOMPLETE_NAMESPACE`` settings variable.
    """
    return getattr(settings, 'AGNOCOMPLETE_NAMESPACE', 'agnocomplete')


def autodiscover():
    """Auto-discover INSTALLED_APPS agnocomplete modules."""
    autodiscover_modules('autocomplete')


default_app_config = 'agnocomplete.app.AgnocompleteConfig'



"""
agnocomplete doc checker
"""
import os
from copy import copy
from os.path import abspath, dirname, join, splitext


def grep(rootdir, searched):
    "grep -l <searched>"
    to_inspect = []
    for root, dirs, files in os.walk(rootdir):
        for _file in files:
            if _file.endswith('.rst'):
                to_inspect.append(join(root, _file))

    to_check = ((_file, open(_file, 'r').read()) for _file in to_inspect)
    to_check = filter(lambda item: searched in item[1], to_check)
    to_check = map(lambda item: item[0], to_check)
    return to_check


def get_expected_html_files(rootdir, files):
    html_files = map(lambda _file: splitext(_file), files)
    html_files = map(lambda item: (item[0].replace(rootdir, '')), html_files)
    html_files = map(lambda item: "{}{}.html".format('/_build/html', item), html_files)
    return set(html_files)


if __name__ == '__main__':
    rootdir = abspath(dirname(__file__))
    files = grep(rootdir, 'literalinclude')
    expected_html_files = get_expected_html_files(rootdir, files)

    # Manually fed:
    html_files = [
        'admin-site.html',
        'autocomplete-definition.html',
        'error-handling.html',
    ]
    # CHECK!
    files = copy(html_files)
    files = map(lambda item: "{}/{}".format("/_build/html", item), files)
    files = set(files)
    diff = expected_html_files.symmetric_difference(files)
    msg = "Have you checked every literalinclude?: {}".format(diff)
    assert files == expected_html_files, msg
    print("Reminder: Check your literalinclude in: {}".format(expected_html_files))
